/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "platform.pmh"

namespace fe
{

bool System::getEnvironmentVariable(const String &name, String &value)
{
	bool rv = false;
#if FE_OS==FE_LINUX || FE_OS==FE_OSX
	char *cp;
	cp = getenv(name.c_str());
	if ((char *) 0 != cp)
	{
		value = cp;
		rv = true;
	}
#elif FE_OS==FE_WIN32 || FE_OS==FE_WIN64
	unsigned long cp;
	const int size = 4096;
	char val[size];
	cp = GetEnvironmentVariable(name.c_str(), val, size);
	if (0 != cp)
	{
		value = val;
		rv = true;
	}
#else
#error
#endif
	return rv;
}

bool System::setCurrentWorkingDirectory(const String &a_path)
{
	bool rv = false;
#if FE_OS==FE_LINUX || FE_OS==FE_OSX
		rv = (0 == chdir(a_path.c_str()));
#elif FE_OS==FE_WIN32 || FE_OS==FE_WIN64
		rv = SetCurrentDirectory(a_path.c_str());
#else
#error
#endif
	return rv;
}

bool System::createDirectory(const String &a_path)
{
	bool rv = false;
#if FE_OS==FE_LINUX || FE_OS==FE_OSX
		rv = !mkdir(a_path.c_str(),0777);
#elif FE_OS==FE_WIN32 || FE_OS==FE_WIN64
		rv = !_mkdir(a_path.c_str());
#else
#error
#endif
	return rv;
}

} /* namespace */

